/*---------------------------------------------------------------------------

    Filename: limitedtimer.js
    Project:  Utilities
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         Extension of the timer class that allows for automatically
                     limiting the number of ticks.
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


// --- sample usage ---
// to see a demonstration of LimitedTimer, uncomment and
// run the following code


/*

// create a new LimitedTimer object
var timer = new LimitedTimer();

// tick every 2 seconds
t.setInterval(2000);

// limit to 3 ticks
t.setMaxTicks(3);

// handle the tick event
t.tick.connect(function() { alert("Ticking..."); });

// start the timer
t.start();

// start the main application loop
Application.run();

*/


// (CLASS) LimitedTimer
// Category: Application
// Description: A class for firing a limited number of events at a specified
//              interval.
// Remarks: The LimitedTimer class represents a timer for firing events at a
//              specified interval with a given limit.
class LimitedTimer
{

    // (EVENT) LimitedTimer.tick
    // Description: Fired each time a timer interval has passed.
    var tick;

    // (CONSTRUCTOR) LimitedTimer.constructor
    // Description: Creates a new LimitedTimer object.
    function LimitedTimer()
    {
        m_timer = new Timer;
        m_timer.tick.connect(this, _handleTick);
        
        tick = new Event;
    }

    // (METHOD) LimitedTimer.getInterval
    // Description: Gets the timer event interval.
    //
    // Syntax: getInterval() : Integer
    //
    // Remarks: This function returns the time interval in milliseconds at
    //          which the timer tick event is to be fired.
    //
    // Returns: The time |interval| in milliseconds at which the the timer tick
    //          event is to be fired.
    function getInterval()
    {
        return m_timer.getInterval();
    }

    // (METHOD) LimitedTimer.isRunning
    // Description: Returns true if the timer is running and false otherwise.
    //
    // Syntax: isRunning() : Boolean
    //
    // Remarks: This function indicates whether or not the timer is currently
    //          running, and therefore, firing timer events. Returns true if
    //          the timer is running and false otherwise. Note the timer will
    //          no longer be running if the specified number of ticks have been
    //          fired.
    //
    // Returns: True if the timer is running and false otherwise.
    function isRunning()
    {
        return m_timer.isRunning();
    }

    // (METHOD) LimitedTimer.setInterval
    // Description: Set the timer event interval.
    //
    // Syntax: setInterval(interval: Integer)
    //
    // Remarks: This function sets the time interval in milliseconds at which
    //          the timer tick event is to be fired.
    //
    // Param(interval): The time |interval| in milliseconds at which the the
    //                  timer tick event is to be fired.
    function setInterval(interval)
    {
        m_timer.setInterval(interval);
    }

    // (METHOD) LimitedTimer.start
    // Description: Starts the timer.
    //
    // Syntax: start()
    //
    // Remarks: This function starts the timer, so that the tick event is fired
    //          at a periodic interval. It will also reset the remaining tick
    //          count.
    function start()
    {
        if (m_maxTicks)
            m_remainingTicks = m_maxTicks;
        m_timer.start();
    }

    // (METHOD) LimitedTimer.stop
    // Description: Stops the timer.
    //
    // Syntax: stop()
    //
    // Remarks: This function stops the timer, so that the tick event stops
    //          firing.
    function stop()
    {
        m_timer.stop();
    }

    // (METHOD) LimitedTimer.setMaxTicks
    // Description: Set the maximum number of tick events.
    //
    // Syntax: setMaxTicks(ticks: Integer)
    //
    // Remarks: This function sets the maximum number of tick events that the
    //          timer will fire.
    //
    // Param(ticks): The maximum number of tick events to fire.
    function setMaxTicks(ticks)
    {
        m_maxTicks = ticks > 0 ? ticks : undefined;
    }

    // (METHOD) LimitedTimer.getMaxTicks
    // Description: Gets the maximum number of tick events.
    //
    // Syntax: getMaxTicks() : Integer
    //
    // Remarks: This function returns the maximum number of tick events that
    //          the timer will fire.
    //
    // Returns: The maximum |number| of tick events that the timer will fire
    //          when running.
    function getMaxTicks()
    {
        return m_maxTicks;
    }

    // (METHOD) LimitedTimer.getRemainingTicks
    // Description: Gets the remaining number of tick events.
    //
    // Syntax: getRemainingTicks() : Integer
    //
    // Remarks: This function returns the remaining number of tick events that
    //          the timer will fire before stopping.
    //
    // Returns: The remaining number of tick events that the timer will fire
    //          before stopping. If the maximum number of tick events has not
    //          been set or the timer is not running, this function will return
    //          |undefined|.
    function getRemainingTicks()
    {
        return m_maxTicks && m_timer.isRunning() ? m_remainingTicks : undefined;
    }


    // --- private member functions ---

    function _handleTick(sender, args)
    {
         if (m_maxTicks)
         {
            if (!m_remainingTicks)
                return;

            m_remainingTicks--;
         }

         tick.fire(sender, args);

        if (m_maxTicks && !m_remainingTicks)
            m_timer.stop();
    }


    // --- private member variables ---

    var m_timer,
        m_maxTicks,
        m_remainingTicks;
}
